﻿import mx.utils.Delegate;
//import mx.format.NumberFormat;
import fluid.CityMarker;
import fluid.MapContainer;
import migurski.gis.AffineTransformer;
import migurski.gis.MapRegistrationMarker;
import migurski.gis.projections.LambertAzimuthalEqualAreaProjection;

/*
 * -----------------------------------------------------------------
 * Copyright (c) 2005 Fluid Inc. All rights reserved.
 * -----------------------------------------------------------------
 *
 * MapDemo.as
 * 
 * @desc The main application class. 
 * 
 * @author Darren David
 *
 * CVS Information:
 * $Id: MapDemo.as,v 1.7 2005/07/08 14:20:58 migurski Exp $
 * 
 * -----------------------------------------------------------------
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * -----------------------------------------------------------------
 */

class fluid.MapDemo
{//}
	//{ Properties	
	private static var _instance : MapDemo;	
	private var _stageClip : MovieClip;	
	//}
	
	// Singleton, use getInstance()
	private function MapDemo()
	{
		// Set this to true to see the behind the scenes math at work		
		_global.SHOW_DEBUG_OUTPUT = false;
	}

	//{ Static Methods
	
	/**
	* Returns the instance of the MapDemo class
	*/
	public static function getInstance() : MapDemo
	{
		if ( MapDemo._instance == undefined )
			MapDemo._instance = new MapDemo();
		
		return MapDemo._instance;
	}
	
	//}


	//{ Public Methods
	
	public function toString() : String
	{
		return "MapDemo[]";
	}
	
	public function init() : Void
	{
		if ( _global.SHOW_DEBUG_OUTPUT ) trace( this + ": Initializing Maps" );

		/*************************************************************
		* STEP 1: Acquire
		* STEP 2: Translate
		* 
		* For the purposes of this demo, we are going to create 3 arrays
		* of simple objects that contain city data, one array for each 
		* map on the stage. This data could easily come in from XML, a
		* Web Service, database, etc., in which case it might need to 
		* be translated into a form that Flash can work with easily.
		*
		***************************************************************/
		
		var lower48MapCityData : Array = 
			[
				{ 
				  name : "San Francisco",
				  lat  : 37.77,
				  long : -122.43 
				},
				
				{ 
				  name : "New York",
				  lat  : 40.75,
				  long : -73.99 
				},
				
				{ 
				  name : "Miami",
				  lat  : 25.77,
				  long : -80.29 
				},

				{ 
				  name : "Seattle",
				  lat  : 47.59,
				  long : -122.29 
				}
			];
		
		var alaskaMapCityData : Array = 
			[
				{ 
				  name : "Anchorage",
				  lat  : 61.18,
				  long : -149.86 
				},
				
				{ 
				  name : "Nome",
				  lat  : 65.16,
				  long : -164.98 
				}
			];

		var hawaiiMapCityData : Array = 
			[
				{ 
				  name : "Honolulu",
				  lat  : 21.31,
				  long : -157.84 
				}
			];
			
			
		// pointers to the map containers placed on the stage
		var maps : Array = [ this.stageClip.mapLower48,
							 this.stageClip.mapAlaska,
							 this.stageClip.mapHawaii ];


		/*************************************************************
		*
		* Before we can transform our points, we need to derive a set of 
		* functions that describe how to take the raw output of the projection
		* and transform the x/y coordinates so that they end up where we
		* want them on the map. We do this by placing 3 MapRegistrationMarkers
		* in our map and assigning the correct properties on each.
		* ( For more info on this see the AffineTransform.registerMarker() method ).
		* Once those have been placed in the right spots in the map movieclip,
		* we feed them all in to the AffineTransformer and call calibrate()
		* which generates our functions for us at runtime.
		* 
		* Each map instance has its own instance of an AffineTransformer 
		* because the transformations may vary by map.
		* 
		* Each map instance also has an instance of the projection class
		* that matches its projection (in this case, our maps use the
		* Lambert Azimuthal Equal Area Projection ). When we instantiate
		* the projection, we pass in the values for latitude and longitude
		* that would represent the center of the projection for your map.
		*
		***************************************************************/

		for ( var i : Number = 0; i < maps.length; i++ )
		{
			var map : MapContainer = MapContainer( maps[i] );
			
			// listen for clicks on each map
			map.addEventListener( "click", Delegate.create( this, this.handleMapClicked ) );
			
			// instantiate the transformer and the projection
			map.transformer = new AffineTransformer();
			map.projection = new LambertAzimuthalEqualAreaProjection( 45, -100 );
			
			// register the markers with the transformer
			for ( var j : Number = 0; j < map.markers.length; j++ )
			{
				try
				{
					map.transformer.registerMarker( MapRegistrationMarker( map.markers[j] ) );
				}
				catch ( e : Error )
				{
					trace("** Registration Error: " + e.message );
				}
			}
			
			// calibrate the transformer
			try
			{
				map.transformer.calibrate();
			}
			catch ( e : Error )
			{
				trace ("** Map Init Error on " + map._name + ": " + e.message );
			}
		}
		
		// now plot the cities
		this.plotCities( this.stageClip.mapLower48, lower48MapCityData );
		this.plotCities( this.stageClip.mapAlaska, alaskaMapCityData );
		this.plotCities( this.stageClip.mapHawaii, hawaiiMapCityData );
	}
		
	//}
	
	
	//{ Private Methods
	
	private function plotCities( map : MovieClip, cityArr : Array )
	{
		for ( var i : Number = 0; i< cityArr.length; i++ )
		{
			var city : Object = cityArr[i];
			var cityMarker : MovieClip;
			
			/*************************************************************
			* STEP 3: Project
			* 
			* For each city, we call the GPSToMapCoords() method in our
			* projection instance.
			*
			***************************************************************/

			var projectedMapCoords : Object = map.projection.GPSToMapCoords( city.lat, city.long );


			/*************************************************************
			* STEP 4: Transform
			* 
			* For each city, we call the transform method on our transformer
			* instance, passing in the /projected/ coordinates from above.
			* 
			***************************************************************/

			var transformedMapCoords : Object = map.transformer.transform( projectedMapCoords.x, projectedMapCoords.y );


			/*************************************************************
			* STEP 5: Plot
			* 
			* Now that we've got our coordinates in the right space,
			* plotting is easy.
			* 
			***************************************************************/
			
			cityMarker = CityMarker( map.attachMovie( "CityMarker", "marker"+i, map.getNextHighestDepth(), { label : city.name } ) );
			cityMarker._x = transformedMapCoords.x;
			cityMarker._y = transformedMapCoords.y;


			if ( _global.SHOW_DEBUG_OUTPUT ) 
			{
				trace("")
				trace(this + ": plotCities(): " +
					" plotted " + city.name + " at " + [ transformedMapCoords.x, transformedMapCoords.y ] +
					" lat=" + city.lat + " long=" + city.long +
					" projected x,y=" + [ projectedMapCoords.x, projectedMapCoords.y ] );
			}
		}
	}
	
	//}

	
	//{ Event Handlers
	
	/**
	* User clicked on the map. 
	*/
	private function handleMapClicked( eventObj : Object ) : Void
	{
		/*************************************************************
		* STEP 6: Interact
		* 
		* This illustrates how we can convert from Flash coordinate
		* space back to GPS coordinates. MapContainer fires a "click"
		* event that contains the location of the click in Flash space.
		* 
		***************************************************************/

		var map : MapContainer = MapContainer( eventObj.target );
		var transformer : AffineTransformer = AffineTransformer( map.transformer );
		
		var untransformedMapCoords : Object = map.transformer.untransform( eventObj.clickX, eventObj.clickY );
		var unprojectedMapCoords : Object = map.projection.MapToGPSCoords( untransformedMapCoords.x, untransformedMapCoords.y );
		
		//var template:String = "-I.dd";
		//var fmt:NumberFormat = new NumberFormat(template);
		//var outputStr : String = map._name + " was clicked at lat/long: " + [ fmt.format(unprojectedMapCoords.lat), fmt.format(unprojectedMapCoords.long) ];
		var outputStr : String = map._name + " was clicked at lat/long: " + [ unprojectedMapCoords.lat, unprojectedMapCoords.long ];
		
		this.stageClip.tfOutput.text = outputStr;
	}
	
	//}
	
	
	//{ Accessors
	
	public function get stageClip() : MovieClip { return this._stageClip; }
	public function set stageClip( mc : MovieClip ) : Void { this._stageClip = mc ; }

	//}
}