﻿import migurski.gis.projections.ILinearProjection;
import migurski.util.MathUtil;

/*
 * -----------------------------------------------------------------
 * Copyright (c) 2005 Michal Migurski. All rights reserved.
 * -----------------------------------------------------------------
 *
 * AlbersConicalEqualAreaProjection.as
 * 
 * @desc calculates AlbersConicalEqualAreaProjection
 *       http://mathworld.wolfram.com/AlbersEqual-AreaConicProjection.html
 *
 * @author Michal Migurski
 * @author Darren David
 *
 * CVS Information:
 * $Id: AlbersConicalEqualAreaProjection.as,v 1.2 2005/07/04 17:32:51 darren Exp $
 * 
 * -----------------------------------------------------------------
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * -----------------------------------------------------------------
 */


class migurski.gis.projections.AlbersConicalEqualAreaProjection
implements ILinearProjection
{//}	
	// these numbers are stored as radians
	private var _lat0 : Number;
	private var _long0 : Number;
	private var _lat1 : Number;
	private var _lat2 : Number;
	
	private var _n : Number;
	private var _C : Number;

	
	/**
	* Constructor
	*/
	public function AlbersConicalEqualAreaProjection( lat0 : Number, long0 : Number, lat1 : Number, lat2 : Number )
	{
		this._lat0 = lat0;
		this._long0 = long0;
		this._lat1 = lat1;
		this._lat2 = lat2;
		
		this._n = 0.5 * ( Math.sin( lat1 ) + Math.sin( lat2 ) );
		this._C = Math.pow(Math.cos(lat1), 2) + (2 * this._n * Math.sin(lat1));
	}
	
	
	/**
	 * Given lat/long in degrees in GPS space, projects point in Map space
	 * @param lat The latitude component of the point in degrees
	 * @param long The longitude component of the point in degrees
	 * @return An Object containing x and y properties
	 */
	public function GPSToMapCoords( lat : Number, long : Number) : Object
	{
		var point : Object = new Object();
		var latRadians : Number = MathUtil.deg2rad( lat );
		var longRadians : Number = MathUtil.deg2rad( long );
		var p, p0, th : Number;
		
		if (this._n == 0) 
		{
			p0 = 0;
			p = 0;
		} 
		else 
		{
			p0 = Math.sqrt(this._C - (2 * this._n * Math.sin(this._lat0))) / this._n;
			p = Math.sqrt(this._C - (2 * this._n * Math.sin(latRadians))) / this._n;
		}

		th = this._n * (longRadians - this._long0);
		
		point.x = p * Math.sin(th);
		point.y = p0 - (p * Math.cos(th));

		return point;
	}
	

	/**
	 * Given x/y in Map space, convert to lat/long in degrees in GPS space
	 * @param lat The x component of the point in pixels
	 * @param long The y component of the point in pixels
	 * @return An Object containing lat and long properties
	 */
	public function MapToGPSCoords( x : Number, y : Number ) : Object
	{
		var coords : Object = new Object();
		var p, p0, th : Number;
		
		if (this._n == 0) 
		{
			p0 = 0;
			p = Math.sqrt(Math.pow(x, 2) + Math.pow((p0 - y), 2));
		} 
		else 
		{
			p0 = Math.sqrt(this._C - (2 * this._n * Math.sin(this._lat0))) / this._n;
			p = Math.sqrt(Math.pow(x, 2) + Math.pow((p0 - y), 2));
		}

		th = Math.atan(x / (p0 - y));
	
		coords.lat = MathUtil.deg2rad( Math.asin((this._C - (Math.pow(p, 2) * Math.pow(this._n, 2))) / (2 * this._n)));
		coords.long = MathUtil.deg2rad( this._long0 + (th / this._n));

		return coords;
	}
}