﻿import migurski.gis.projections.ILinearProjection;
import migurski.util.MathUtil;

/*
 * -----------------------------------------------------------------
 * Copyright (c) 2005 Michal Migurski. All rights reserved.
 * -----------------------------------------------------------------
 *
 * MercatorProjection.as
 * 
 * @desc calculates MercatorProjection
 *       http://mathworld.wolfram.com/MercatorProjection.html
 *
 * @author Michal Migurski
 * @author Darren David
 *
 * CVS Information:
 * $Id: MercatorProjection.as,v 1.2 2005/07/04 17:32:51 darren Exp $
 * 
 * -----------------------------------------------------------------
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * -----------------------------------------------------------------
 */


class migurski.gis.projections.MercatorProjection
implements ILinearProjection
{//}	
	/**
	* Constructor
	*/
	public function MercatorProjection()
	{
	}
	
	
	/**
	 * Given lat/long in degrees in GPS space, projects point in Map space
	 * @param lat The latitude component of the point in degrees
	 * @param long The longitude component of the point in degrees
	 * @return An Object containing x and y properties
	 */
	public function GPSToMapCoords( lat : Number, long : Number) : Object
	{
		var point : Object = new Object();
		var latRadians : Number = MathUtil.deg2rad( lat );
		var longRadians : Number = MathUtil.deg2rad( long );
		
		point.x = longRadians;
		point.y = Math.log( Math.tan( ( Math.PI / 4 ) + ( latRadians / 2 )) );
		
		return point;
	}
	

	/**
	 * Given x/y in Map space, convert to lat/long in degrees in GPS space
	 * @param lat The x component of the point in pixels
	 * @param long The y component of the point in pixels
	 * @return An Object containing lat and long properties
	 */
	public function MapToGPSCoords( x : Number, y : Number ) : Object
	{
		var coords : Object = new Object();
		
		coords.lat  = MathUtil.rad2deg( 2 * (Math.atan(Math.pow(10, y)) - (Math.PI / 4)) );
		coords.long = MathUtil.rad2deg( x );

		return coords;
	}
}